/*
	Sample source code for Lasso C API SDK

	This example creates an LCAPI module for Lasso Professional 6.x which adds
	three new substitution tags to LDML: [example_math_abs], [example_math_sin],
	and [example_math_sqrt].  The word "example" has been added to the
	beginning of these tag names so that they do not interfere with the
	built-in [math_abs], [math_sin], and [math_sqrt] tags.
*/
#include "MathFuncsCAPI.h"
#include	<stdio.h>	//	for scanf()
#include	<fcntl.h>
#include	<math.h>
#include	<time.h>

void registerLassoModule( void )
{
	lasso_registerTagModule( "Math", "example_math_abs",	tagMathAbsFunc, flag_typeSubstitutionTag, "Absolute value" );
	lasso_registerTagModule( "Math", "example_math_sin",	tagMathSinFunc, flag_typeSubstitutionTag, "Trigonometric Sine function" );
	lasso_registerTagModule( "Math", "example_math_sqrt",	tagMathSqrtFunc, flag_typeSubstitutionTag, "Square Root" );
}

//
//	Handle [Example_Math_Abs: -45.2] tag
//
osError
tagMathAbsFunc( lasso_request_t token, tag_action_t action )
{
	short	err = osErrNoErr;

	if ( action == tagFormatSubstitution )
	{
		auto_lasso_value_t	v;
		double				tempValue;
		char				tempText[32];

		lasso_getTagParam( token, 0, &v );
		sscanf( v.name, "%lf", &tempValue );
		sprintf( tempText, "%.15lg", (double)fabs( tempValue ) );
		lasso_outputTagData( token, tempText );
	}
	lasso_setResultCode( token, err );
	return err;
}

//
//	Handle [Example_Math_Sin: -45.2] tag
//
osError
tagMathSinFunc( lasso_request_t token, tag_action_t action )
{
	short	err = osErrNoErr;

	if ( action == tagFormatSubstitution )
	{
		auto_lasso_value_t	v;
		double				tempValue;
		char				tempText[32];

		lasso_getTagParam( token, 0, &v );
		sscanf( v.name, "%lf", &tempValue );
		sprintf( tempText, "%.15lg", (double)sin( tempValue ) );
		lasso_outputTagData( token, tempText );
	}
	lasso_setResultCode( token, err );
	return err;
}

//
//	Handle [Example_Math_Sqrt: 125] tag
//
osError
tagMathSqrtFunc( lasso_request_t token, tag_action_t action )
{
	short	err = osErrNoErr;

	if ( action == tagFormatSubstitution )
	{
		auto_lasso_value_t	v;
		double				tempValue;
		char				tempText[32];

		lasso_getTagParam( token, 0, &v );
		sscanf( v.name, "%lf", &tempValue );
		sprintf( tempText, "%.15lg", (double)sqrt( tempValue ) );
		lasso_outputTagData( token, tempText );
	}
	lasso_setResultCode( token, err );
	return err;
}
