<?LassoScript
// Define tags for use below.

//[Include_Safe] performs an include without returning a no records found error.
define_tag: 'include_safe';
	protect;
		return: (include: (params)->(get:1));
	/protect;
/define_tag;

define_tag: 'ldml_nocache';
	$__http_header__ += 'Expires: Mon, 26 Jul 1997 05:00:00 GMT\r\n';
	$__http_header__ += 'Last-Modified: ' + (Date_LocaltoGMT: date )->(format:'%a, %d %m %Y %H:%M:%S') + ' GMT\r\n';
	$__http_header__ += 'Cache-Control: no-store, no-cache, must-revalidate\r\n';
	$__http_header__ += 'Cache-Control: post-check=0, pre-check=0\r\n';
	$__http_header__ += 'Pragma: no-cache\r\n';
/define_tag;

define_tag: 'variable_default';
	precondition: (params)->(size) > 0;
	precondition: (params)->(get:1)->(type) == 'string';
	if: !(var_defined: (params)->(get:1)) || ((var: (params->(get:1))) == Null);
		var: (params)->(get:1) = '';
	/if;
	if: ((var: (params->(get:1))) == '') && ((params)->(size) > 1);
		loop: -loopfrom=2, -loopto=(params)->(size);
			local: 'temp'=(params)->(get:loop_count);
			if: #temp!=null && #temp!='';
				var: (params)->(get:1) = #temp;
			/if;
		/loop;
	/if;
/define_tag;

define_tag: 'code_highlight';
	precondition: (params)->(size) == 2;
	local: 'output'=(params)->(get:1);
	local: 'tag'=(params)->(get:2);
	if: #tag->(beginswith:'-');
		#output = (string_replaceregexp: #output, -search='(' + #tag + ')(\\W)', -replace='<b>\\1</b>\\2', -ignorecase);
	else: #tag >> '->';
		local: 'index'=#tag->(find: '->');
		local: 'index2'=#tag->(find: ']');
		#tag = #tag->(substring: (#index + 2), (#index2 - #index - 2));
		#output = (string_replaceregexp: #output, -search='([>\\(;])(' + #tag + ')([:\\]\\)])', -replace='\\1<b>\\2</b>\\3', -ignorecase);
	else: #tag->(beginswith:'[');
		local: 'index'=#tag->(find: '[');
		local: 'index2'=#tag->(find: ']');
		#tag = #tag->(substring: (#index + 1), (#index2 - #index - 1));
		#output = (string_replaceregexp: #output, -search='(' + #tag + ')([:\\]\\)])', -replace='<b>\\1</b>\\2', -ignorecase);
		#output = (string_replaceregexp: #output, -search='(/' + #tag + ')([:\\]\\)])', -replace='<b>\\1</b>\\2', -ignorecase);
	/if;

	#output = (string_replaceregexp: #output, -search='\\r\\n|\\r|\\n', -replace='<br>', -ignorecase);	
	#output = (string_replaceregexp: #output, -search='<\\?LassoScript', -replace='&lt;?LassoScript', -ignorecase);
	#output = (string_replaceregexp: #output, -search='<(/?a|/?form)', -replace='&lt;\\1', -ignorecase);

	// Also fix breaking problems in supported browsers.
	#output = (string_replaceregexp: #output, -search='(\\-[a-zA-Z]+)', -replace='<wbr><nobr>\\1</nobr>');

	return: #output;
/define_tag;

define_tag: 'code_cleanup';
	precondition: (params)->(size) == 1;
	local: 'output'=(params)->(get:1);
	#output = (string_replace: -find='&nbsp;', -replace=' ', #output);
	#output = (string_replace: -find='<br>', -replace='\\r', #output);
	#output = (string_replace: -find='<i>', -replace='', #output);
	#output = (string_replace: -find='</i>', -replace='', #output);
	#output = (string_replace: -find='&lt;', -replace='<', #output);
	#output = (string_replace: -find='&gt;', -replace='>', #output);
	#output = (string_replace: -find='<!--', -replace='', #output);
	#output = (string_replace: -find='-->', -replace='', #output);

	#output = (encode_html: #output);

	#output = (string_replace: -find='href', -replace='&#104;&#114;&#101;&#102;', #output);
	return: #output;
/define_tag;

define_tag: 'example_search';
	precondition: (params)->(size) == 1;
	local: 'output'=(params)->(get:1);
	#output = (string_replace: -find=' ', -replace='&nbsp;', #output);
	#output = (string_replace: -find='<', -replace='&lt;', #output);
	#output = (string_replace: -find='>', -replace='&gt;', #output);
	return: #output;
/define_tag;

define_tag: 'list_cleanup';
	precondition: (params)->(size) == 1;
	local: 'output'=(params)->(get:1);

	if: #output >> '...';
		local: 'index'=#output->(find: '...');
		#output = #output->(substring: (1), (#index + 2));
	/if;

	return: #output;
/define_tag;

define_tag: 'status_cleanup';
	precondition: (params)->(size) == 1;
	local: 'output'=(params)->(get:1);

	#output = (string_replace: #output, -find=',', -replace=',<br>');
	#output = (string_replace: #output, -find='MySQL', -replace='MySQL');
	#output = (string_replace: #output, -find='FMP', -replace='FileMaker');

	return: #output;
/define_tag;

define_tag: 'short_cleanup';
	precondition: (params)->(size) == 2;
	local: 'output'=(string: (params)->(get:1));
	local: 'size'=(integer: (params)->(get:2));

	if: #output >> '...';
		local: 'index'=#output->(find: '...');
		#output = #output->(substring: (1), (#index + 2));
	/if;

	if: #output->length < #size;
		return: #output;
	/if;

	#output = #output->(substring:0,(#size - 3));
	if: (#output->(endswith: '...'));
		return: #output;
	else: (#output->(endswith: '..'));
		return: #output + '.';
	else: (#output->(endswith: '.'));
		return: #output + '..';
	else;
		return: #output + '...';
	/if;
/define_tag;


// Define Global Variables

var: 'ref_tag_inline'=(array:-Database='LDML7_Reference',-Layout='Tags',-KeyField='tag_id');
var: 'ref_tag_index'='tag_id';
var: 'ref_tag_name'='tag_name';
var: 'ref_tag_description'='tag_description';
var: 'ref_tag_changenotes'='tag_changenotes';
var: 'ref_tag_syntax'='tag_syntax';
var: 'ref_tag_category'='tag_category';
var: 'ref_tag_type'='tag_type';
var: 'ref_tag_language'='tag_language';
var: 'ref_tag_support'='tag_support';
var: 'ref_tag_edition'='tag_edition';
var: 'ref_tag_version'='tag_version';
var: 'ref_tag_change'='tag_history';
var: 'ref_tag_platform'='tag_platform';
var: 'ref_tag_datasource'='tag_datasource';
var: 'ref_tag_pagenumber'='tag_pagenumber';
var: 'ref_tag_basic'='tag_basic';
var: 'ref_tag_display'='tag_display';
var: 'ref_tag_equiv'='tag_equiv';
var: 'ref_tag_output'='tag_output';
var: 'ref_tag_security'='tag_security';
var: 'ref_tag_implementation'='tag_implementation';
var: 'ref_tag_sourceavailable'='tag_sourceavailable';

var: 'ref_param_inline'=(array:-Database='LDML7_Reference',-Layout='Parameters',-KeyField='param_id');
var: 'ref_param_tag_index'='tag_id';
var: 'ref_param_name'='param_name';
var: 'ref_param_description'='param_description';
var: 'ref_param_type'='param_type';
var: 'ref_param_required'='param_required';
var: 'ref_param_optional'='param_optional';

var: 'ref_example_inline'=(array:-Database='LDML7_Reference',-Layout='Examples',-KeyField='ex_id');
var: 'ref_example_tag_index'='tag_id';
var: 'ref_example_tag_name'='tag_name';
var: 'ref_example_title'='ex_title';
var: 'ref_example_description'='ex_description';
var: 'ref_example_example'='ex_example';
var: 'ref_example_result'='ex_result';

var: 'ref_comments_inline'=(array:-Database='LDML7_Reference',-Layout='Comments',-KeyField='id');
var: 'ref_comments_tag_index'='tag_id';
var: 'ref_comments_tag_name'='tag_name';
var: 'ref_comments_author'='com_author';
var: 'ref_comments_email'='com_email';
var: 'ref_comments_subject'='com_subject';
var: 'ref_comments_comments'='com_comments';
var: 'ref_comments_datetime'='com_date';
var: 'ref_comments_timestamp'='com_stamp';

var: 'ref_join_inline'=(array:-Database='LDML7_Reference',-Layout='RelatedTags',-KeyField='id');
var: 'ref_join_tag_index'='tag_id';
var: 'ref_join_related_index'='rel_id';
var: 'ref_join_related_type'='rel_type';
var: 'ref_join_related_name'='rel_name';


// Create Session
session_start: -name='_reference', -expires=1440, -uselink;
session_addvar: -name='_reference', 'ref_date';
session_addvar: -name='_reference', 'ref_time';

if: (!var_defined: 'ref_date');
	var: 'ref_date' = (server_date: -short);
	var: 'ref_time' = (server_time: -long);
/if;


// Change all form params to variables
html_comment;
search_arguments;
	var: (search_fielditem) = (search_valueitem);
	output: '\n<br>' + (encode_html: search_fielditem) + ' = ' + (encode_html: search_valueitem), -encodenone;
/search_arguments;
/html_comment;


// Define Standard Lists

session_addvar: -name='_reference', 'admin_category_list';
session_addvar: -name='_reference', 'admin_legacy_list';

if: !(var_defined: 'admin_category_list') || ((var: 'admin_category_list')->type != 'array') || ((var: 'admin_category_list')->size <= 0);
	var: 'admin_category_list' = (array);
	inline: $ref_tag_inline, -FindAll, -Distinct, $ref_tag_display='Y', $ref_tag_basic='Y', -ReturnField=$ref_tag_category, -SortField=$ref_tag_category, -MaxRecords='all';
		records;
			var: 'temp' = (string: field: $ref_tag_category);
			$temp->trim;
			$admin_category_list->(insert: $temp);
		/records;
	/inline;
/if;

if: !(var_defined: 'admin_legacy_list') || ((var: 'admin_legacy_list')->type != 'array') || ((var: 'admin_legacy_list')->size <= 0);
	var: 'admin_legacy_list' = (array);
	inline: $ref_tag_inline, -FindAll, -Distinct, $ref_tag_display='Y', $ref_tag_basic='N', -ReturnField=$ref_tag_category, -SortField=$ref_tag_category, -MaxRecords='all';
		records;
			var: 'temp' = (string: field: $ref_tag_category);
			$temp->trim;
			$admin_legacy_list->(insert: $temp);
		/records;
	/inline;
/if;


// Set up the Tab Environment

encode_set: -encodenone;

if:(var_defined:'admin_path')==false;
	var:'admin_path'='';
/if;

if:(var_defined:'admin_file')==false;
	var:'admin_file'=(LassoApp_Link: 'default.lasso');
/if;

if:(var_defined:'admin_pane')==false;
	var:'admin_pane'=(LassoApp_Link: 'default.lasso');
/if;

var: 'arrow_graphic'=(LassoApp_Link: 'images/arrow.gif');


var: 'tab1_enabled'='#999999';
var: 'tab1_enabled_right'=(LassoApp_Link: 'images/tab1_enabled_right.gif');
var: 'tab1_enabled_left'=(LassoApp_Link: 'images/tab1_enabled_left.gif');
var: 'tab1_disabled'='#CCCCCC';
var: 'tab1_disabled_right'=(LassoApp_Link: 'images/tab1_disabled_right.gif');
var: 'tab1_disabled_left'=(LassoApp_Link: 'images/tab1_disabled_left.gif');

var: 'tab2_enabled'='#EEEEEE';
var: 'tab2_enabled_right'=(LassoApp_Link: 'images/tab2_enabled_right.gif');
var: 'tab2_enabled_left'=(LassoApp_Link: 'images/tab2_enabled_left.gif');
var: 'tab2_disabled'='#999999';
var: 'tab2_disabled_right'='';
var: 'tab2_disabled_left'='';


var:'admin_tab1'=(form_param:'tab1');
var:'admin_tab2'=(form_param:'tab2');

if: (var: 'tag') != '';
	var:'admin_tab1'='detail';
	var:'admin_tab2'='tag';
	var:'detail_index'=(var: 'tag');
	var:'detail_mode'='detail_related';
/if;

if:(var:'admin_tab1')=='';
	var:'admin_tab1'='search';
/if;

	var:'admin_tab1_array'=(array:
		(map:
			'name'='search',
			'label'='Search',
			'color'=$tab1_disabled,
			'default'='search',
			'active'=true,
			'tab2_array'=(array:
				(map:
					'name'='search',
					'label'='Basic',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'search/basic/default.lasso'),
					'library'=(LassoApp_Link: 'search/basic/library.lasso')
				),(map:
					'name'='advanced',
					'label'='Advanced',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'search/advanced/default.lasso'),
					'library'=(LassoApp_Link: 'search/advanced/library.lasso')
				),(map:
					'name'='comments',
					'label'='Comments',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'search/comments/default.lasso'),
					'library'=(LassoApp_Link: 'search/comments/library.lasso')
				),(map:
					'name'='examples',
					'label'='Examples',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'search/examples/default.lasso'),
					'library'=(LassoApp_Link: 'search/examples/library.lasso')
				)
			)
		),(map:
			'name'='browse',
			'label'='Browse',
			'color'=$tab1_disabled,
			'default'='category',
			'active'=true,
			'tab2_array'=(array:
				(map:
					'name'='category',
					'label'='Category',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'browse/category/default.lasso'),
					'library'=(LassoApp_Link: 'browse/category/library.lasso')
				),(map:
					'name'='legacy',
					'label'='Legacy',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'browse/legacy/default.lasso'),
					'library'=(LassoApp_Link: 'browse/legacy/library.lasso')
				)
			)
		),(map:
			'name'='detail',
			'label'='Detail',
			'color'=$tab1_disabled,
			'default'='tag',
			'active'=true,
			'tab2_array'=(array:
				(map:
					'name'='tag',
					'label'='Tag',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'detail/tag/default.lasso'),
					'library'=(LassoApp_Link: 'detail/tag/library.lasso')
				),(map:
					'name'='comments',
					'label'='Comments',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'detail/comments/default.lasso'),
					'library'=(LassoApp_Link: 'detail/comments/library.lasso')
				),(map:
					'name'='code',
					'label'='Code',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'detail/code/default.lasso'),
					'library'=(LassoApp_Link: 'detail/code/library.lasso')
				)
			)
		),(map:
			'name'='list',
			'label'='List',
			'color'=$tab1_disabled,
			'default'='category',
			'active'=true,
			'tab2_array'=(array:
				(map:
					'name'='category',
					'label'='Preferred',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'list/category/default.lasso'),
					'library'=(LassoApp_Link: 'list/category/library.lasso')
				),(map:
					'name'='legacy',
					'label'='Legacy',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'list/legacy/default.lasso'),
					'library'=(LassoApp_Link: 'list/legacy/library.lasso')
				),(map:
					'name'='tag',
					'label'='All',
					'color'=$tab2_disabled,
					'active'=true,
					'format'=(LassoApp_Link: 'list/tag/default.lasso'),
					'library'=(LassoApp_Link: 'list/tag/library.lasso')
				)
			)
		)
	);


// Set up the default tab positions

loop:(var:'admin_tab1_array')->(size);
	if:(var:'admin_tab1_array')->(get:loop_count)->(find:'name')==(var:'admin_tab1');
		var:'admin_tab1_label'=(var:'admin_tab1_array')->(get:loop_count)->(find:'label');

		(var:'admin_tab1_array')->(get:loop_count)->(insert:'color'=$tab1_enabled);
		(var:'admin_tab1_array')->(get:loop_count)->(insert:'right'=$tab1_enabled_right);
		(var:'admin_tab1_array')->(get:loop_count)->(insert:'left'=$tab1_enabled_left);

		var:'admin_tab2_array'=(var:'admin_tab1_array')->(get:loop_count)->(find:'tab2_array');
		if:(var:'admin_tab2')=='';
			var:'admin_tab2'=(var:'admin_tab1_array')->(get:loop_count)->(find:'default');
		/if;
		(var:'admin_tab1_array')->(get:loop_count)->(insert:'default'=$admin_tab2);
	else;
		(var:'admin_tab1_array')->(get:loop_count)->(insert:'color'=$tab1_disabled);
		(var:'admin_tab1_array')->(get:loop_count)->(insert:'right'=$tab1_disabled_right);
		(var:'admin_tab1_array')->(get:loop_count)->(insert:'left'=$tab1_disabled_left);
	/if;
/loop;

if:(var:'admin_tab2_array')==null;
	(var:'admin_tab2_array')=(var:'admin_tab1_array')->(get:1)->(find:'tab2_array');
/if;

loop:(var:'admin_tab2_array')->(size);
	if:(var:'admin_tab2_array')->(get:loop_count)->(find:'name')==(var:'admin_tab2');
		var:'admin_tab2_label'=(var:'admin_tab2_array')->(get:loop_count)->(find:'label');
		var:'admin_librarypath'=(var:'admin_tab2_array')->(get:loop_count)->(find:'library');
		var:'admin_panepath'=(var:'admin_tab2_array')->(get:loop_count)->(find:'format');

		(var:'admin_tab2_array')->(get:loop_count)->(insert:'color'=$tab2_enabled);
		(var:'admin_tab2_array')->(get:loop_count)->(insert:'right'=$tab2_enabled_right);
		(var:'admin_tab2_array')->(get:loop_count)->(insert:'left'=$tab2_enabled_left);
	else;
		(var:'admin_tab2_array')->(get:loop_count)->(insert:'color'=$tab2_disabled);
		(var:'admin_tab2_array')->(get:loop_count)->(insert:'right'=$tab2_disabled_right);
		(var:'admin_tab2_array')->(get:loop_count)->(insert:'left'=$tab2_disabled_left);
	/if;
/loop;

/encode_set;

?>

[if: $admin_librarypath != null]
[html_comment] Library [var: 'admin_librarypath'] [/html_comment]
[include: $admin_librarypath]
[html_comment] /Library [var: 'admin_librarypath'] [/html_comment]
[/if]
