<?LassoScript

//
// Define the Lasso Administration script
//

Log_Warning: 'Lasso Administration: Initializing';

// Check if the table exists
Local: 'table_exists' = false;
Inline: -Database='lasso_admin', -SQL='SHOW COLUMNS FROM admin_prefs', -MaxRecords='all';
	if: (found_count) > 0;
		#table_exists = true;
	/if;
/inline;
if: (#table_exists == false);
	// Create table if it doesn't exist already
	Inline: -Database='lasso_admin',
		-sql='CREATE TABLE IF NOT EXISTS lasso_admin.admin_prefs
			(
				id INT PRIMARY KEY AUTO_INCREMENT,
				type VARCHAR(255),
				database_name VARCHAR(255),
				table_name VARCHAR(255),
				pref_name VARCHAR(255),
				pref_value LONGBLOB
			)';
	/Inline;
/if;

//
// Define Lasso Administration Helper Tags
//

define_tag: '_admin_internaldatabase', -priority='replace';
	return: 'lasso_internal';
/define_tag;

define_tag:'_admin_refreshsecurity', -priority='replace';
	_security_flushcaches;
	var: '__admin_connectors_array' = null;
	var: '__admin_connector_id' = null;
	var: '__admin_hosts_array' = null;
	var: '__admin_host_id' = null;
	var: '__admin_databases_array' = null;
	var: '__admin_database_id' = null;
	var: '__admin_tables_array' = null;
	var: '__admin_table_id' = null;
	var: '__admin_fields_array' = null;
	var: '__admin_groups_array' = null;
	var: '__admin_users_array' = null;
/define_tag;

define_tag:'_admin_listconnectors', -priority='replace';
	auth_admin;
	if: ((var: '__admin_connectors_array') != null) && ((var: '__admin_connectors_array')->size > 0);
		return: (var: '__admin_connectors_array');
	/if;

	// Find friendly connector names
	local: 'connector_map' = (map:
		'internalmysqlds'='Lasso Connector for Lasso MySQL',
		'fmremotedatasource'='Lasso Connector for FileMaker Pro',
		'fmserveradvanceddatasource'='Lasso Connector for FileMaker SA',
		'mysqlds'='Lasso Connector for MySQL',
	);
	inline: -database='lasso_internal', -table='global_prefs', 'store_key'='_admin_connector_alias:', -search, -maxrecords='all';
		records;
			local: 'temp_name' = (string: (field: 'store_key'))->(substring: 25);
			local: 'temp_alias' = (string: (field: 'data'));
			#connector_map->(insert: #temp_name = #temp_alias);
		/records;
	/inline;

	local: 'output' = (array);
	inline: -search, -database='lasso_internal', -table='security_datasources', -op='neq', 'name'='_email_sender',
		-returnfield='name', -returnfield='id', -returnfield='enabled', -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#temp->(insert: 'alias'=(_admin_ifnull: #connector_map->(find: (field: 'name')), (field: 'name')));
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_connectors_array' = #output;
	var: '__admin_hosts_array' = null;
	var: '__admin_databases_array' = null;
	var: '__admin_tables_array' = null;
	var: '__admin_fields_array' = null;
	return: #output;
/define_tag;

define_tag:'_admin_updateconnector', -required='connector', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, -database='lasso_internal', -table='security_datasources', -keyfield='id', -keyvalue=#connector, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_defaultconnector', -required='id', -priority='replace';
	auth_admin;
	if: ((var: '__admin_connectors_array') == null) || ((var: '__admin_connectors_array')->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = (var: '__admin_connectors_array')->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_connector_id') && ((var: '__admin_connector_id') > 0);
		local: 'temp' = (var: '__admin_connectors_array')->(find: (var: '__admin_connector_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: ((var: '__admin_connectors_array')->size > 0);
		return: (var: '__admin_connectors_array')->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_refreshconnector', -required='connector', -priority='replace';
	auth_admin;
	if: (#connector) == 1;
		inline: -database='lasso_internal', -sql='flush tables';
		/inline;
	/if;
	_datasource_reload: (integer: #connector);
/define_tag;

define_tag:'_admin_listhosts', -required='connector', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_hosts_array') != null) && ((var: '__admin_connector_id') == #connector);
		return: (var: '__admin_hosts_array');
	/if;

	// Find friendly host names
	local: 'host_map' = (map: 
		'1'='Lasso MySQL',
	);
	inline: -database='lasso_internal', -table='global_prefs', 'store_key'='_admin_host_alias:', -search, -maxrecords='all';
		records;
			local: 'temp_name' = (string: (field: 'store_key'))->(substring: 19);
			local: 'temp_alias' = (string: (field: 'data'));
			#host_map->(insert: #temp_name = #temp_alias);
		/records;
	/inline;
	
	inline: -search, -database='lasso_internal', -table='security_datasource_hosts', -op='eq', 'id_datasource'=#connector, -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			#temp->(insert: 'alias'=(_admin_ifnull: #host_map->(find: (field: 'id')), ((field: 'name') + ':' + (field: 'port'))));
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_hosts_array' = #output;
	var: '__admin_connector_id' = #connector;
	var: '__admin_databases_array' = null;
	var: '__admin_tables_array' = null;
	var: '__admin_fields_array' = null;
	return: #output;
/define_tag;

define_tag:'_admin_defaulthost', -required='id', -priority='replace';
	auth_admin;
	if: ((var: '__admin_hosts_array') == null) || ((var: '__admin_hosts_array')->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = (var: '__admin_hosts_array')->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_host_id') && ((var: '__admin_host_id') > 0);
		local: 'temp' = (var: '__admin_hosts_array')->(find: (var: '__admin_host_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: ((var: '__admin_hosts_array')->size > 0);
		return: (var: '__admin_hosts_array')->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_updatehost', -required='host', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, -database='lasso_internal', -table='security_datasource_hosts', -keyfield='id', -keyvalue=#host, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletehost', -required='host', -priority='replace';
	auth_admin;
	inline: -delete, -database='lasso_internal', -table='security_datasource_hosts', -keyfield='id', -keyvalue=#host;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;

	local: 'databases'=(_admin_listdatabases: #host);
	loop: #databases->size;
		_admin_deletedatabase: #databases->(get: loop_count)->first;
	/loop;
/define_tag;

define_tag:'_admin_listdatabases', -required='host', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_databases_array') != null) && ((var: '__admin_host_id') == #host);
		return: (var: '__admin_databases_array');
	/if;
	inline: -search, -database='lasso_internal', -table='security_datasource_hosts', -keyfield='id', -keyvalue=#host;
		if: (string: (field: 'enabled')) >> 'Y';
			local: 'enabled' = 'Y';
		else;
			local: 'enabled' = 'N';
		/if;
	/inline;
	inline: -search, -database='lasso_internal', -table='security_datasource_databases', -op='eq', 'id_host'=#host, -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		if: #enabled >> 'Y';
			local: 'live' = (_database_names: (field: 'id_datasource'), (field: 'id_host'));
		else;
			local: 'live' = (array);
		/if;
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: #live->(find: (field: 'name')) != (array);
				if: (field: 'enabled') == 'Y';
					#temp->(insert: 'status'='Enabled');
				else: (field: 'enabled') == 'N';
					#temp->(insert: 'status'='Disabled');
				else;
					#temp->(insert: 'status'='New');
				/if;
			else;
				#temp->(insert: 'status'='Not Available');
			/if;
			if: #temp->(find: 'alias') == '';
				#temp->(insert: 'alias'=(field: 'name'));
			/if;
			if: ((field: 'alias') == '') || ((field: 'alias') == (field: 'name'));
				#temp->(insert: 'label'=(field: 'name'));
			else;
				#temp->(insert: 'label'=(field: 'name') + ' (' + (field: 'alias') + ')');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_databases_array' = #output;
	var: '__admin_host_id' = #host;
	var: '__admin_tables_array' = null;
	var: '__admin_fields_array' = null;
	return: #output;
/define_tag;

define_tag:'_admin_defaultdatabase', -required='id', -required='array', -priority='replace';
	auth_admin;
	if: !(local_defined: 'array');
		local: 'array' = (var: '__admin_databases_array');
	/if;
	if: (#array == null) || (#array->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = #array->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_database_id') && ((var: '__admin_database_id') > 0);
		local: 'temp' = #array->(find: (var: '__admin_database_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (#array->size > 0);
		return: #array->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_updatedatabase', -required='database', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -keyvalue=#database, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletedatabase', -required='database', -priority='replace';
	auth_admin;
	inline: -delete, -database='lasso_internal', -table='security_datasource_databases', -keyfield='id', -keyvalue=#database;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;

	local: 'tables'=(_admin_listtables: #database);
	loop: #tables->size;
		_admin_deletetable: #tables->(get: loop_count)->first;
	/loop;
/define_tag;

define_tag:'_admin_listtables', -required='database', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_tables_array') != null) && ((var: '__admin_database_id') == #database);
		return: (var: '__admin_tables_array');
	/if;

	local: 'db' = (var: '__admin_databases_array')->(find: #database);
	if: (#db != null) && (#db->size > 0);
		local: 'database_alias' = #db->(get:1)->second->(find: 'alias');
		local: 'database_status' = #db->(get:1)->second->(find: 'status');
		if: #database_status >> 'Enabled';
			inline: -show, -database=#database_alias; /inline;
		/if;
	/if;

	inline: -search, -database='lasso_internal', -table='security_database_tables', -op='eq', 'id_database'=#database, -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			if: #temp->(find: 'alias') == '';
				#temp->(insert: 'alias'=(field: 'name'));
			/if;
			if: ((field: 'alias') == '') || ((field: 'alias') == (field: 'name'));
				#temp->(insert: 'label'=(field: 'name'));
			else;
				#temp->(insert: 'label'=(field: 'name') + ' (' + (field: 'alias') + ')');
			/if;
			if: (local_defined: 'database_alias') && (#database_alias != '');
				#temp->(insert: 'database_alias' = #database_alias);
			/if;
			if: (local_defined: 'database_status') && (#database_status != '');
				#temp->(insert: 'database_status' = #database_status);
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_tables_array' = #output;
	var: '__admin_database_id' = #database;
	var: '__admin_fields_array' = null;
	return: #output;
/define_tag;

define_tag:'_admin_defaulttable', -required='id', -required='array', -priority='replace';
	auth_admin;
	if: !(local_defined: 'array');
		local: 'array' = (var: '__admin_tables_array');
	/if;
	if: (#array == null) || (#array->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = #array->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_table_id') && ((var: '__admin_table_id') > 0);
		local: 'temp' = #array->(find: (var: '__admin_table_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (#array->size > 0);
		return: #array->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_updatetable', -required='table', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, -database='lasso_internal', -table='security_database_tables', -keyfield='id', -keyvalue=#table, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletetable', -required='table', -priority='replace';
	auth_admin;
	inline: -delete, -database='lasso_internal', -table='security_database_tables', -keyfield='id', -keyvalue=#table;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;

	local: 'fields'=(_admin_listfields: #table);
	loop: #fields->size;
		_admin_deletefield: #fields->(get: loop_count)->first;
	/loop;
/define_tag;

define_tag:'_admin_listfields', -required='table', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_fields_array') != null) && ((var: '__admin_table_id') == #table);
		return: (var: '__admin_fields_array');
	/if;

	local: 'tb' = (var: '__admin_tables_array')->(find: #table);
	if: (#tb != null) && (#tb->size > 0);
		local: 'database_alias' = #tb->(get:1)->second->(find: 'database_alias');
		local: 'table_alias' = #tb->(get:1)->second->(find: 'alias');
		local: 'database_status' = #tb->(get:1)->second->(find: 'database_status');
		if: #database_status >> 'Enabled';
			inline: -show, -database=#database_alias, -table=#table_alias; /inline;
		/if;
	/if;

	inline: -search, -database='lasso_internal', -table='security_table_fields', -op='eq', 'id_table'=#table, -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_fields_array' = #output;
	var: '__admin_table_id' = #table;
	return: #output;
/define_tag;

define_tag:'_admin_defaultfield', -required='id', -priority='replace';
	auth_admin;
	if: ((var: '__admin_fields_array') == null) || ((var: '__admin_fields_array')->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = (var: '__admin_fields_array')->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (var_defined: '__admin_field_id') && ((var: '__admin_field_id') > 0);
		local: 'temp' = (var: '__admin_fields_array')->(find: (var: '__admin_field_id'));
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: ((var: '__admin_fields_array')->size > 0);
		return: (var: '__admin_fields_array')->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_updatefield', -required='field', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);

	loop: #temp->size;
		if: #temp->(get: loop_count)->type == 'pair';
			#params->(insert: #temp->(get: loop_count));
		/if;
	/loop;

	inline: -update, -database='lasso_internal', -table='security_table_fields', -keyfield='id', -keyvalue=#field, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletefield', -required='field', -priority='replace';
	auth_admin;
	inline: -delete, -database='lasso_internal', -table='security_table_fields', -keyfield='id', -keyvalue=#field;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_listgroups', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_groups_array') != null);
		return: (var: '__admin_groups_array');
	/if;

	inline: -findall, -database='lasso_internal', -table='security_groups', -sortfield='name', -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_groups_array' = #output;
	return: #output;
/define_tag;

define_tag:'_admin_defaultgroup', -required='id', -required='array', -priority='replace';
	auth_admin;
	if: !(local_defined: 'array');
		local: 'array' = (var: '__admin_groups_array');
	/if;
	if: (#array == null) || (#array->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = #array->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (#array->size > 0);
		return: #array->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_addgroup', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	if: #params !>> 'id_site';
		#params->(insert: 'id_site'=1);
	/if;
	if: #params !>> 'created';
		#params->(insert: 'created'=date->(format: '%Q %T'));
	/if;
	inline: -add, -database='lasso_internal', -table='security_groups', #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_updategroup', -required='group', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	inline: -update, -database='lasso_internal', -table='security_groups', -keyfield='id', -keyvalue=#group, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deletegroup', -required='group', -priority='replace';
	auth_admin;
	inline: -delete, -database='lasso_internal', -table='security_groups', -keyfield='id', -keyvalue=#group;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
	inline: -database='lasso_internal', -sql='DELETE FROM security_ug_map WHERE id_group = ' + #group;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
	inline: -database='lasso_internal', -sql='DELETE FROM security_group_db_map WHERE id_group = ' + #group;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
	inline: -database='lasso_internal', -sql='DELETE FROM security_group_file_perms WHERE id_group = ' + #group;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
	inline: -database='lasso_internal', -sql='DELETE FROM security_group_generic_perms WHERE id_group = ' + #group;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
	inline: -database='lasso_internal', -sql='DELETE FROM security_group_host_map WHERE id_group = ' + #group;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
	inline: -database='lasso_internal', -sql='DELETE FROM security_group_table_map WHERE id_group = ' + #group;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
	inline: -database='lasso_internal', -sql='DELETE FROM security_group_tag_map WHERE id_group = ' + #group;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;


define_tag:'_admin_listusers', -priority='replace';
	auth_admin;
	local: 'output' = (array);
	if: ((var: '__admin_users_array') != null);
		return: (var: '__admin_users_array');
	/if;

	inline: -findall, -database='lasso_internal', -table='security_users', -sortfield='name', -MaxRecords='all';
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
		records;
			local: 'temp' = (map);
			loop: (field_name: -count);
				#temp->(insert: (field_name: loop_count) = (field: (field_name: loop_count)));
			/loop;
			if: (field: 'enabled') == 'Y';
				#temp->(insert: 'status'='Enabled');
			else: (field: 'enabled') == 'N';
				#temp->(insert: 'status'='Disabled');
			/if;
			#output->(insert: (field: 'id') = #temp);
		/records;
	/inline;
	var: '__admin_users_array' = #output;
	return: #output;
/define_tag;

define_tag:'_admin_defaultuser', -required='id', -required='array', -priority='replace';
	auth_admin;
	if: !(local_defined: 'array');
		local: 'array' = (var: '__admin_users_array');
	/if;
	if: (#array == null) || (#array->type != 'array');
		return: null;
	/if;
	if: (local_defined: 'id') && (#id > 0);
		local: 'temp' = #array->(find: #id);
		if: #temp->size > 0;
			return: #temp->(get:1);
		/if;
	/if;
	if: (#array->size > 0);
		return: #array->(get:1);
	/if;
	return: null;
/define_tag;

define_tag:'_admin_adduser', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	if: #params !>> 'id_site';
		#params->(insert: 'id_site'=1);
	/if;
	if: #params !>> 'created';
		#params->(insert: 'created'=date->(format: '%Q %T'));
	/if;
	inline: -add, -database='lasso_internal', -table='security_users', #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_updateuser', -required='user', -priority='replace';
	auth_admin;
	local: 'temp' = (params);
	local: 'params' = (array);
	iterate: #temp, (local: 'item');
		if: #item->type == 'pair';
			#params->(insert: #item);
		/if;
	/iterate;
	inline: -update, -database='lasso_internal', -table='security_users', -keyfield='id', -keyvalue=#user, #params;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag:'_admin_deleteuser', -required='user', -priority='replace';
	auth_admin;
	inline: -delete, -database='lasso_internal', -table='security_users', -keyfield='id', -keyvalue=#user;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
	inline: -database='lasso_internal', -sql='DELETE FROM security_ug_map WHERE id_user = ' + #user;
		var: '_error' = (error_currenterror);
		var: '_code' = (error_currenterror: -errorcode);
	/inline;
/define_tag;

define_tag: '_admin_servicepath', -priority='replace';
	return: (string: admin_lassoservicepath) + '/';
/define_tag;

?>
