<?LassoScript

Define_Tag: '_file_probeEOL',
	-required='buff',
	-returntype='bytes',
	-priority='replace';

	iterate:#buff, local:'char';

		if:#char == 0x000A;
			return:bytes:'\n';
		else if:#char == 0x000D;
			if:#buff->get(loop_count + 1) == 0x000A; // \r\n
				return:bytes:'\r\n';
			else; // \r
				return:bytes:'\r';
			/if;
		/if;

	/iterate;

	// if we can't probe for the EOL, we return the platform
	// specific EOL, rather than returning null, which is bad.
	if: (lasso_version: -lassoplatform) == 'win32';
		return:bytes:'\r\n';
	else;
		return:bytes:'\n'; // osx and linux both use \n line endings
	/if;

/Define_Tag;


Define_Tag: 'File_GetLineCount',
	-Required = 'File',
	-Optional = 'FileEndOfLine',
	-Priority = 'Replace';

	local:'buff' = file_read:#file;
	if:file_currenterror(-errorcode) != 0 || #buff->size == 0;
		return:0;
	/if;

	local:'myeol'=null;
	if:!local_defined('fileendofline');
		#myeol = _file_probeEOL:#buff;
	else;
		#myeol = bytes:#fileendofline;

		// if the bytes conversion is wrong, then we need to throw an error
		// otherwise, we'll get an empty search, and an infinite loop!
		fail_if: #myeol->size == 0, -1, 'File_GetLineCount: Invalid EOL value';

	/if;
	local:'count' = 1; // at least one line
	local:'lastPos' = #buff->find(#myeol);

	while:#lastPos != 0;
		#lastPos = #buff->find(#myeol, #lastPos + #myeol->size);
		#count += 1;
		fail_if: loop_count > #buff->size, -1, 'File_GetLineCount: Loop count exceded the bounds of the buffer.';
	/while;

	return:#count;

/Define_Tag;


Define_Tag: 'File_ReadLine',
	-Required='File',
	-Required='FileLineNumber',
	-Optional='FileEndOfLine',
	-Priority='Replace';

	// contents of file (this is probably not the most efficient thing to do)
	local: 'b' = (file_read: #file);
	if: ((file_currenterror: -errorcode) != 0) || (#b->size == 0);
		return: 0;
	/if;

	// end of line character
	local: 'eol' = null;
	if: ! (local_defined: 'fileendofline');
		#eol = (_file_probeEOL: #b);
	else;

		#eol = (bytes: #fileendofline);

		// if the bytes conversion is wrong, then we need to throw an error
		// otherwise, we'll get an empty search, and an infinite loop!
		fail_if: #eol->size == 0, -1, 'File_GetLineCount: Invalid EOL value';

	/if;

	// line we are looking for
	local: 'l' = (integer: #FileLineNumber);
	if: #l == 0;
		return: string;
	/if;

	// previos position
	local: 'pp' = 1;

	// position
	local: 'p' = #b->(find: #eol);

	if: #p == 0;
		return: (string: #b);
	else;

		local: 'c' = 0;

		while: #pp - 1 <= #b->size;
			#c += 1;

			if: #l == #c;
				loop_abort;
			/if;
	
			#pp = #p + #eol->size;
			#p = #b->(find: #eol, #pp);
	
			if: #p == 0;
				#p = #b->size + 1;
			/if;
	
		/while;

		if: #l > #c;
			return: string;
		/if;
	
		protect;
			return: #b->(getrange: #pp, #p - #pp);
		/protect;

	/if;

/Define_Tag;


//
// [File_Chmod]
//
// [File_Chmod: 'filename', Decimal Value]
// [File_Chmod: 'filename', Hexadecimal Value]
// [File_Chmod: 'filename', 'Quoted Octal Value']
// [File_Chmod: 'filename', -a='rwxst']
// [File_Chmod: 'filename', -a='rwxst', -u='rwx', -g='rwx', -o='rwx']
//

define_tag: 'file_chmod',
		-required='file',
		-optional='a',
		-optional='u',
		-optional='g',
		-optional='o';

	if: !(local_defined: 'a');
		local: 'a' = '';
	/if;

	local: 'perms' = 0;

	if: #a->(isa: 'string') && ((integer: #a) > 0);

		local: 'pow' = (integer: (math_pow: 8, #a->size - 1));
		iterate: #a, (local: 'char');
			#perms += (#pow * (integer: #char));
			#pow /= 8;
		/iterate;

	else: #a->(isa: 'integer');

		local: 'perms' = #a;

	else;
		if: (#a >> 'r');
			#perms += 292; // 0444
		/if;
		if: (#a >> 'w');
			#perms += 146; // 0222
		/if;
		if: (#a >> 'x');
			#perms += 73; // 0111
		/if;
		if: (#a >> 's');
			#perms += 3072; // 6000
		/if;
		if: (#a >> 't');
			#perms += 512; // 1000
		/if;
		if: (local_defined: 'u');
			if: (#u >> 'r') && (#a !>> 'r');
				#perms += 256; // 0400
			/if;
			if: (#u >> 'w') && (#a !>> 'w');
				#perms += 128; // 0200
			/if;
			if: (#u >> 'x') && (#a !>> 'x');
				#perms += 64; // 0100
			/if;
		/if;
		if: (local_defined: 'g');
			if: (#g >> 'r') && (#a !>> 'r');
				#perms += 32; // 0040
			/if;
			if: (#g >> 'w') && (#a !>> 'w');
				#perms += 16; // 0020
			/if;
			if: (#g >> 'x') && (#a !>> 'x');
				#perms += 8; // 0010
			/if;
		/if;
		if: (local_defined: 'o');
			if: (#o >> 'r') && (#a !>> 'r');
				#perms += 4; // 0004
			/if;
			if: (#o >> 'w') && (#a !>> 'w');
				#perms += 2; // 0002;
			/if;
			if: (#o >> 'x') && (#a !>> 'x');
				#perms += 1; // 0001
			/if;
		/if;
	/if;

	_file_chmod: #file, #perms;

/define_tag;

?>