<?LassoScript

// [..._Serve] tags.
//
// These tags serve a 'file' in place of the output of the current format file.  They can be used instead
// of a [Redirect_URL] to serve a different file, can be used to serve a PDF or XML file created in memory,
// or can be used to serve an image file from a database field.
//
// Each tag sets the contents of the page in __http_header__ to an appropriate value and then resets the
// headers of the page with the file name and MIME type.  Finally, [Abort] is called to prevent further
// processing of the page.
//
// 'content' - The first parameter is the content of the file to be served.
// -File - Optional parameter which specifies the name of the file to be served.
// -Type - Optional parameter which specifies the MIME type of the content to be served.

Define_Tag: 'File_Serve', -Priority='Replace', -Required='content', -Optional='file', -Optional='type';
	Variable: '__http_header__' = '';
	$__http_header__ += 'HTTP/1.0 200 OK\r\n';
	$__http_header__ += 'Server: Lasso/' + (lasso_version: -lassoversion) + '\r\n';
	$__http_header__ += 'MIME-Version: 1.0\r\n';
	if: (local_defined: 'type');
		$__http_header__ += 'Content-Type: ' + (string: #type) + '\r\n';
	else;
		$__http_header__ += 'Content-Type: text/html\r\n';
	/if;
	if: (local_defined: 'file') && (#file != '');
		local: 'safefile' = #file;
		#safefile->(replace: '"', '\\"');
		$__http_header__ += 'Content-Disposition: attachment; filename="' + #safefile + '"\r\n';
	/if;
	$__html_reply__ = #content;
	$__http_header__ += 'Content-Length: ' + $__html_reply__->size + '\r\n';
	Abort;
/Define_Tag;

Define_Tag: 'XML_Serve', -Priority='Replace', -Required='content', -Optional='file', -Optional='type';
	If: !(Local_Defined: 'file');
		Local: 'file' = '';
	/if;
	if: !(Local_Defined: 'type');
		Local: 'type' = 'text/xml';
	/if;
	File_Serve: -content=(string: #content), -file=(string: #file), -type=(string: #type);
/Define_Tag;

Define_Tag: 'PDF_Serve',
	-Priority='Replace',
	-Required='content',
	-Type = 'PDF_Doc',
	-Optional='file',
	-Optional='type';
	If: !(Local_Defined: 'file');
		Local: 'file' = '';
	/if;
	if: !(Local_Defined: 'type');
		Local: 'type' = 'application/pdf';
	/if;
	File_Serve: -content=#content->data, -file=(string: #file), -type=(string: #type);
/Define_Tag;

/**************************************************
*
* Author: Adam Randall
* Company: OmniPilot Software
* Date: 1/14/2004
* Version: 1.0
*
* This tag will allow you to stream a very large
* file, while utilizing a very small amount of
* resources. Load on the server should be minimal
*
**************************************************/

define_tag: 'file_stream',
	-required = 'file',
	-optional = 'name',
	-optional = 'type',
	-priority = 'replace';

	lasso_executiontimelimit: 0;

	local:
		'file_size' = (file_getsize: #file),
		'buffer' = 30000;

	if: !(local_defined: 'type');
		local: 'type' = 'application/octet-stream';
	/if;

	if: !(local_defined: 'name');
		local: 'safename' = #file->(split: '/')->last;
	else;
		local: 'safename' = #name;
	/if;
	#safename->(replace: '"', '\\"');

	if: (file_currenterror) == 'No error';

		$__http_header__ = 'HTTP/1.0 200 OK\r\n';
		$__http_header__ += 'Server: Lasso/' + (lasso_version: -lassoversion)->(get: 1) + '\r\n';
		$__http_header__ += 'MIME-Version: 1.0\r\n';
		$__http_header__ += 'Content-Type: ' + #type + '\r\n';
		$__http_header__ += 'Content-Disposition: attachment; filename="' + #safename + '"\r\n';
		$__http_header__ += 'Content-Length: ' + #file_size + '\r\n';

	loop:
		-loopfrom = 0,
		-loopto = ((math_ceil: #file_size / (decimal: #buffer)) - 1),
		-loopincrement = 1;

			local: 'start' = (loop_count) * #buffer;
			local: 'end' = #start + #buffer;

			if: #end > #file_size;
				#end = #file_size;
			/if;

			$__html_reply__ = (file_read:
				#file,
				-filestartpos = #start,
				-fileendpos = #end);

			server_push;

		/loop;

		server_push; // just in case
		abort;

	else;
		fail: -1, 'No permission to file';
	/if;

/define_tag;

?>
